# Testing basic integration of the package into Plone
from zope.component import getUtility

from plone.portlets.interfaces import IPortletType

from collective.carousel.tests.base import TestCase


class CarouselTestCase(TestCase):

    def test_carousel_layer_available(self):
        self.failUnless('carousel' in self.portal.portal_skins.objectIds())

    def test_carousel_layer_in_stack(self):
        self.skins = self.portal.portal_skins
        theme = self.skins.getDefaultSkin()
        path = self.skins.getSkinPath(theme)
        path = [x.strip() for x in path.split(',')]
        self.failUnless('carousel' in path)
        self.assertEqual(path[1], 'carousel')

    def test_js_available(self):
        jsreg = getattr(self.portal, 'portal_javascripts')
        script_ids = jsreg.getResourceIds()
        self.failUnless('++resource++plone.app.jquerytools.plugins.js'
                        in script_ids)
        self.failUnless('jquery.tools.min.js' not in script_ids)
        self.failUnless('carousel.js' in script_ids)

    def test_css_available(self):
        cssreg = getattr(self.portal, 'portal_css')
        stylesheets_ids = cssreg.getResourceIds()
        self.failUnless('carousel.css' in stylesheets_ids)

    def test_collections_carouselable(self):
        from collective.carousel.interfaces import ICarouselProvider
        self.setRoles('Manager', )

        self.folder.invokeFactory('Collection', 'test-collection')
        carouselable_col = getattr(self.folder, 'test-collection')
        self.failUnless(ICarouselProvider.providedBy(carouselable_col))

    def test_portlet_type_registered(self):
        portlet = getUtility(IPortletType, name='portlet.Carousel')
        self.assertEquals(portlet.addview, 'portlet.Carousel')


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from zope.component import adapts
from zope.interface import implements
from zope.i18nmessageid import MessageFactory

from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
                                         ReferenceBrowserWidget
from Products.ATContentTypes.interface import IATContentType
from Products.Archetypes.public import ReferenceField

from archetypes.schemaextender.interfaces import ISchemaExtender, \
                                                 IBrowserLayerAwareExtender
from archetypes.schemaextender.field import ExtensionField

from collective.carousel.interfaces import ICollectiveCarouselLayer

_ = MessageFactory('collective.carousel')


class CarouselProviderField(ExtensionField, ReferenceField):
    """A field, storing reference to an object, providing content for carousel.
    """


class ContentTypeExtender(object):
    adapts(IATContentType)
    implements(ISchemaExtender,
               IBrowserLayerAwareExtender)
    layer = ICollectiveCarouselLayer

    _fields = [
        CarouselProviderField("carouselprovider",
            schemata = "settings",
            # the field accepts Collections only atm.
            # Would be cool to have ATRBW to be able to select items by
            # interface with something like 'allowed_interfaces'
            allowed_types = ('Topic', 'Collection',),
            relationship = 'Carousel',
            languageIndependent = True,
            multiValued = True,
            widget = ReferenceBrowserWidget(
                label = _(u"label_carouselprovider_title",
                    default=u"Carousel object."),
                description = _(u"help_carouselprovider",
                    default=u"Object providing items (content) for carousel."),
                ),
            ),
        ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self._fields



# Carousel is rendered through a viewlet in IAboveContent
# using items provided by the carousel provider added to the context
from plone.app.portlets.storage import PortletAssignmentMapping
from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletRenderer
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility, getMultiAdapter

from collective.carousel.portlets import carousel
from collective.carousel.tests.base import TestCase

# default test query
query = [{
    'i': 'portal_type',
    'o': 'plone.app.querystring.operation.selection.is',
    'v': ['Document', 'Event', 'News Item']
}]


class PortletTest(TestCase):

    def afterSetUp(self):
        """Set up the carousel Collection and some dummy objects"""

        self.setRoles('Manager')
        self.folder.invokeFactory('Collection', 'collection')
        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)

        field = self.folder.Schema().getField('carouselprovider')
        field.set(self.folder, collection)

        # add a few objects
        self.folder.invokeFactory('Document', 'carousel-doc')
        self.folder.invokeFactory('News Item', 'carousel-news-item')
        self.folder.invokeFactory('Event', 'carousel-event')

    def testPortletTypeRegistered(self):
        portlet = getUtility(IPortletType, name='portlet.Carousel')
        self.assertEquals(portlet.addview, 'portlet.Carousel')

    def testInterfaces(self):
        portlet = carousel.Assignment(header=u"title")
        self.failUnless(IPortletAssignment.providedBy(portlet))
        self.failUnless(IPortletDataProvider.providedBy(portlet.data))

    def testInvokeAddview(self):
        portlet = getUtility(IPortletType, name='portlet.Carousel')
        mapping = self.portal.restrictedTraverse(
            '++contextportlets++plone.leftcolumn')
        for m in mapping.keys():
            del mapping[m]
        addview = mapping.restrictedTraverse('+/' + portlet.addview)
        addview.createAndAdd(data={'header': u"test title"})
        self.assertEquals(len(mapping), 1)
        self.failUnless(isinstance(mapping.values()[0], carousel.Assignment))

    def testInvokeEditView(self):
        mapping = PortletAssignmentMapping()
        request = self.folder.REQUEST
        mapping['foo'] = carousel.Assignment(header=u"title")
        editview = getMultiAdapter((mapping['foo'], request), name='edit')
        self.failUnless(isinstance(editview, carousel.EditForm))

    def testRenderer(self):
        context = self.folder
        request = self.folder.REQUEST
        view = self.folder.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.rightcolumn',
                             context=self.portal)
        assignment = carousel.Assignment(header=u"title")

        renderer = getMultiAdapter((context, request, view,
                                    manager, assignment),
                                   IPortletRenderer)
        self.failUnless(isinstance(renderer, carousel.Renderer))


class TestRenderer(TestCase):

    # default test query
    query = [{
        'i': 'portal_type',
        'o': 'plone.app.querystring.operation.selection.is',
        'v': ['Document', 'Event', 'News Item']
    }]

    def afterSetUp(self):
        self.setRoles(('Manager', ))
        self.folder.invokeFactory('Collection', 'collection')
        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)

        field = self.folder.Schema().getField('carouselprovider')
        field.set(self.folder, collection)

        # add a few objects
        self.folder.invokeFactory('Document', 'carousel-doc')
        self.folder.invokeFactory('News Item', 'carousel-news-item')
        self.folder.invokeFactory('Event', 'carousel-event')

    def renderer(self, context=None, request=None, view=None,
                 manager=None, assignment=None):
        context = context or self.folder
        request = request or self.folder.REQUEST
        view = view or self.folder.restrictedTraverse('@@plone')
        manager = manager or getUtility(IPortletManager,
                                        name='plone.rightcolumn',
                                        context=self.portal)
        assignment = assignment or carousel.Assignment(header=u"title")

        return getMultiAdapter((context, request, view, manager, assignment),
                               IPortletRenderer)

    def test_render(self):
        r = self.renderer(context=self.portal,
                          assignment=carousel.Assignment(
                          header=u"title",
                          target_collection=
                          '/plone/Members/test_user_1_/collection'))
        r = r.__of__(self.folder)
        r.update()
        output = r.render()
        self.assertTrue('title' in output)

    def test_css_class(self):
        r = self.renderer(
            context=self.portal,
            assignment=carousel.Assignment(header=u"Test carousel"))
        self.assertEquals('portlet-carousel-test-carousel', r.css_class())

    def test_hideheader(self):
        r = self.renderer(
            context=self.portal,
            assignment=carousel.Assignment(header=u"Test carousel", hideheader=True))
        output = r.render()
        self.failUnless('class="portletHeader hiddenStructure"' in output)

    def test_portlet_collection(self):

        query = [{
            'i': 'portal_type',
            'o': 'plone.app.querystring.operation.selection.is',
            'v': ['Document']
        }]
        # add a few documents
        for i in range(6):
            self.folder.invokeFactory('Document', 'document_%s' % i)
            getattr(self.folder, 'document_%s' % i).reindexObject()

        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)

        # the documents are returned by the collection
        collection_num_items = len(self.folder.collection.queryCatalog())
        # We better have some documents - we should have 8
        self.failUnless(collection_num_items >= 8)

        mapping = PortletAssignmentMapping()
        mapping['foo'] = carousel.Assignment(
            header=u"Test carousel",
            target_collection='/Members/test_user_1_/collection')
        r = self.renderer(context=None, request=None, view=None,
                          manager=None, assignment=mapping['foo'])

        # sanity check
        self.assertEqual(r.collection().id, 'collection')

        # we want the portlet to return us the same results as the collection
        self.assertEquals(collection_num_items, len(r.results()))

    def test_edit_link(self):
        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)
        mapping = PortletAssignmentMapping()
        mapping['foo'] = carousel.Assignment(
            header=u"Test carousel",
            target_collection='/Members/test_user_1_/collection')
        r = self.renderer(context=None, request=None, view=None,
                          manager=None, assignment=mapping['foo'])
        self.assertTrue(r.editCarouselLink().endswith('/edit'))


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from zope.component import queryMultiAdapter
from zope.interface import alsoProvides, noLongerProvides
from Products.Five import zcml

from collective.carousel.tests.base import TestCase
from collective.carousel.testing import ICustomType
import collective.carousel


# default test query
query = [{
    'i': 'portal_type',
    'o': 'plone.app.querystring.operation.selection.is',
    'v': ['Document', 'Event', 'News Item']
}]


class ViewsTestCase(TestCase):

    def afterSetUp(self):
        """Set up the carousel Collection and some dummy objects"""

        self.setRoles('Manager')
        self.folder.invokeFactory('Collection', 'collection')
        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)

        field = self.folder.Schema().getField('carouselprovider')
        field.set(self.folder, collection)

        # add a few objects
        self.folder.invokeFactory('Document', 'carousel-doc')
        self.folder.invokeFactory('News Item', 'carousel-news-item')
        self.folder.invokeFactory('Event', 'carousel-event')

    def test_ct_views(self):
        zcml.load_config('testing.zcml', collective.carousel)

        # test tile for a Page
        obj = getattr(self.folder, 'carousel-doc')
        tile = queryMultiAdapter((obj, self.app.REQUEST), name="carousel-view")
        self.failUnless('PAGE' in tile(), obj)

        # test tile for a News Item
        obj = getattr(self.folder, 'carousel-news-item')
        tile = queryMultiAdapter((obj, self.app.REQUEST), name="carousel-view")
        self.failUnless('NEWS ITEM' in tile(), obj)

        # test tile for an Event.
        # Since we don't have any special view registered for Event in
        # zcml we should get default view for an object of this content type
        obj = getattr(self.folder, 'carousel-event')
        tile = queryMultiAdapter((obj, self.app.REQUEST), name="carousel-view")
        self.failUnless('DEFAULT' in tile(), obj)

        # test tile for a Page in a carousel portlet
        # since we don't have any special view for Page in the carousel portlet
        # we should get default tile for the carousel portlet that is the same
        # as for a carousel in the viewlet
        obj = getattr(self.folder, 'carousel-doc')
        tile = queryMultiAdapter((obj, self.app.REQUEST),
                                 name="carousel-portlet-view")
        self.failUnless('<p>This is a DEFAULT tile</p>' in tile(), obj)

    def test_custom_tile(self):
        # re-test tile for an Event.
        # Since we don't have any special view registered for Event in
        # zcml we should get default view for an object of this content type
        # And since we override default tile here we should have custom tile
        # returned.
        obj = getattr(self.folder, 'carousel-event')

        alsoProvides(obj, ICustomType)

        tile = queryMultiAdapter((obj, self.app.REQUEST), name="carousel-view")
        self.failUnless('CUSTOM DEFAULT' in tile(), obj)

        # We revert event to standard state (without custom tile registration)
        noLongerProvides(obj, ICustomType)

        # We should get our DEFAULT tile again
        tile = queryMultiAdapter((obj, self.app.REQUEST), name="carousel-view")
        self.failUnless('DEFAULT' in tile(), obj)


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



# In these tests we are testing a ReferenceField injected to all
# content types. The field has to accept objects, providing
# content for carousel.
#
# We accept only Collections in ATReferenceBrowserWidget of the field
# but in these tests we assume that widget itself works as it should
# and doesn't allow to add objects of any other content type.

from collective.carousel.tests.base import TestCase


class FieldTestCase(TestCase):

    def test_field_available(self):
        # Test that we have a field on objects
        self.folder.invokeFactory('Document', 'my-page')
        new_obj = getattr(self.folder, 'my-page')
        # first test newly created document
        self.failUnless('carouselprovider' in new_obj.Schema().keys())
        # now test the folder
        self.failUnless('carouselprovider' in self.folder.Schema().keys())

    def test_field_stored(self):
        # Whether we can change the field and the value of it is getting stored
        self.setRoles('Manager', )

        self.folder.invokeFactory('Collection', 'test-collection')
        carouselable_col = getattr(self.folder, 'test-collection')
        self.folder.invokeFactory('Document', 'my-page',
                                  carouselprovider=(carouselable_col, ))
        new_obj = getattr(self.folder, 'my-page')
        field = new_obj.Schema().getField('carouselprovider')
        # we deal with multiValued field, thus we are getting a list out of the
        # field
        self.assertEqual(field.get(new_obj), [carouselable_col])


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from Products.CMFCore.utils import getToolByName

def run_all_import_steps(context):
    context = getToolByName(context, "portal_setup")
    import pdb; pdb.set_trace()
    context.runAllImportStepsFromProfile(
        'profile-collective.carousel:default',
        purge_old=False)



from Acquisition import aq_inner
from zope.component import getUtility
from zope.component import getMultiAdapter
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.interfaces import IPloneSiteRoot
from collective.contentleadimage.config import IMAGE_FIELD_NAME, IMAGE_CAPTION_FIELD_NAME
from collective.contentleadimage.leadimageprefs import ILeadImagePrefsForm
from collective.contentleadimage.browser.viewlets import LeadImageViewlet


class LeadImageTile(BrowserView):

    template = ViewPageTemplateFile('templates/lead_image_tile.pt')
    render = template

    @property
    def prefs(self):
        portal = getUtility(IPloneSiteRoot)
        return ILeadImagePrefsForm(portal)

    def tag(self, css_class='tileImage'):
        """ return a tag for the leadimage"""
        context = aq_inner(self.context)

        field = context.getField(IMAGE_FIELD_NAME)
        if field is not None:
            if field.get_size(context) != 0:
                #scale = self.prefs.body_scale_name
                scale = 'leadimage'
                return field.tag(context, scale=scale, css_class=css_class)

        if getattr(context, 'tag', None) is not None:
            return context.tag(scale='mini', css_class=css_class)

        return ''

    def caption(self):
        context = aq_inner(self.context)
        field = context.getField(IMAGE_CAPTION_FIELD_NAME)
        if field is None:
            return ''

        return context.widget(IMAGE_CAPTION_FIELD_NAME, mode='view')

    def isAllowed(self):
        context = aq_inner(self.context)
        portal_type = getattr(context, 'portal_type', None)
        if portal_type in self.prefs.allowed_types:
            return super(LeadImageViewlet, self).render()
        else:
            return ''

    def modified(self):
        """        http://svn.plone.org/svn/plone/Plone/trunk/Products/CMFPlone/browser/ploneview.py
        @return: Last modified as a string, local time format        """
        # Get Plone helper view
        # which we use to convert the date to local format
        plone = getMultiAdapter((self.context, self.request), name="plone")
        time = self.context.modified()
        return plone.toLocalizedTime(time)

    def published(self):
        """        http://svn.plone.org/svn/plone/Plone/trunk/Products/CMFPlone/browser/ploneview.py
        @return: Last modified as a string, local time format        """
        # Get Plone helper view
        # which we use to convert the date to local format
        plone = getMultiAdapter((self.context, self.request), name="plone")
        time = self.context.effective()
        return plone.toLocalizedTime(time)

    def __call__(self):
        return self.render()



from Products.Five import zcml
from Products.Five import fiveconfigure
# from Products.PloneTestCase.PloneTestCase import installPackage
# from collective.testcaselayer.ptc import BasePTCLayer, ptc_layer

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

from zope.interface import Interface


@onsetup
def setup_product():
    """Set up additional products and ZCML required to test this product.

    The @onsetup decorator causes the execution of this body to be deferred
    until the setup of the Plone site testing layer.
    """

    # Load the ZCML configuration for this package and its dependencies

    fiveconfigure.debug_mode = True
    from collective import carousel
    zcml.load_config('testing.zcml', package=carousel)
    zcml.load_config('custom_tile_testing.zcml', package=carousel)
    fiveconfigure.debug_mode = False

    # We need to tell the testing framework that these products
    # should be available. This can't happen until after we have loaded
    # the ZCML.

    ztc.installPackage('collective.carousel')


class ICustomType(Interface):
    """Custom helping interface for testing purposes only!
       Used to provide custom tile registration. Doing this with zcml for real
       is painful since custom zcml registration leaks to the following tests
       lower in the stack. Most probably you never want to use such marker
       interface in real-life implementation and would register your custom
       tile to either IATContentType (tile default for all types) or any
       specific interface like IATNewsItem
    """
    pass


# The order here is important: We first call the deferred function and then
# let PloneTestCase install it during Plone site setup

setup_product()
ptc.setupPloneSite(products=['collective.carousel'])



from zope.interface import Interface


class ICollectiveCarouselLayer(Interface):
    """ A layer specific to collective.carousel
    """


class ICarouselProvider(Interface):
    """ We should be able to mark objects that can be used in carousel with
        the special marker. For now this marks Collections only.
    """

ILeadImageable = None

try:
    from collective.contentleadimage.interfaces import ILeadImageable
except:
    pass

from Products.ATContentTypes.interface import IATContentType
from Products.ATContentTypes.interface import IATNewsItem

if ILeadImageable is not None:
    # Need to make sure we come up top
    class IATWithLeadImage(ILeadImageable, IATContentType):
        """ More specific interface """

    class IATNewsItemWithLeadImage(ILeadImageable, IATNewsItem):
        """ More specific interface """



from zope.interface import alsoProvides
from AccessControl import SecurityManagement

from Products.ATContentTypes.permission import ChangeTopics
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

try:
    from plone.app.collection.interfaces import ICollection
except ImportError:
    from zope.interface import Interface

    class ICollection(Interface):
        pass

from plone.app.layout.viewlets.common import ViewletBase
from plone.app.layout.globals.interfaces import IViewView


class CarouselViewlet(ViewletBase):
    index = ViewPageTemplateFile('templates/carousel.pt')

    def update(self):
        if IViewView.providedBy(self.__parent__):
            alsoProvides(self, IViewView)

    def getProviders(self):
        schema = getattr(self.context, 'Schema', None)
        if schema is None:
            return None
        field = schema().getField('carouselprovider')
        if field is None:
            return None
        return field.get(self.context)

    def results(self, provider):
        results = []
        if provider is not None:
            # by default we assume that only Collections are addable
            # as a carousel provider

            # It doesn't make sense to show *all* objects from a collection
            # - some of them might return hundreeds of objects
            if ICollection.providedBy(provider):
                res = provider.results(b_size=7)
                return res
            return provider.queryCatalog()[:7]
        return results

    def canSeeEditLink(self, provider):
        smanager = SecurityManagement.getSecurityManager()
        return smanager.checkPermission(ChangeTopics, provider)

    def editCarouselLink(self, provider):
        if provider is not None:
            if ICollection.providedBy(provider):
                return provider.absolute_url() + '/edit'
            return provider.absolute_url() + '/criterion_edit_form'
        return None

    def get_tile(self, obj):
        # note to myself
        # When adapter is uesd this means we check whether obj has any special
        # instructions about how to be handled in defined view or interface
        # for multi adapter the same is true except more object than just the
        # obj are check for instructions
        #have to use traverse to make zpt security work
        tile = obj.unrestrictedTraverse("carousel-view")
        if tile is None:
            return None
        return tile()



from zope.i18nmessageid import MessageFactory
from zope.interface import implements
from zope import schema
from zope.formlib import form
from AccessControl import SecurityManagement
from zope.component import getMultiAdapter, getUtility

from plone.memoize.instance import memoize
from plone.i18n.normalizer.interfaces import IIDNormalizer

from Products.ATContentTypes.permission import ChangeTopics
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

# from plone.portlet.collection import collection as base
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base

from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget
from plone.app.vocabularies.catalog import SearchableTextSourceBinder

from Products.ATContentTypes.interface import IATTopic

try:
    from plone.app.collection.interfaces import ICollection
    from plone.app.querystring import queryparser
except ImportError:
    from zope.interface import Interface
    class ICollection(Interface):
        pass

_ = MessageFactory('collective.carousel')


class ICarouselPortlet(IPortletDataProvider):
    """A portlet displaying a carousel with a Collection's results
    """

    header = schema.TextLine(
        title=_(u"Portlet header"),
        description=_(u"Title of the rendered portlet"),
        required=True)

    hideheader = schema.Bool(
        title=_(u"Don't show the header."),
        description=_(u"If enabled, the portlet will not show the header."),
        required=False,
        default=False)

    target_collection = schema.Choice(
        title=_(u"Target collection"),
        description=_(u"Find the collection which provides the items to list"),
        required=True,
        source=SearchableTextSourceBinder(
            {'portal_type': ('Topic', 'Collection')},
            default_query='path:'))

    limit = schema.Int(
        title=_(u"Limit"),
        description=_(u"Specify the maximum number of items to show in the "
                      u"portlet. Leave this blank to show all items."),
        required=False)

    hide_controls = schema.Bool(
        title=_(u"Hide controls"),
        description=_(u"Tick this box if you want to temporarily hide "
                      "carousel controls i.e. next, prev, pause buttons."),
        required=True,
        default=False)

    timer = schema.Int(
        title=_(u"Timer"),
        description=_(u"How fast the carousel should be rotated \
                        (seconds)"),
        required=False,
        default=10)


class Assignment(base.Assignment):
    implements(ICarouselPortlet)

    header = u""
    hideheader = False
    target_collection = None
    limit = None
    hide_controls = False
    timer = 10

    def __init__(self, header=u"", hideheader=False, target_collection=None, limit=None,
                 hide_controls=False, timer=10):
        self.header = header
        self.hideheader = hideheader
        self.target_collection = target_collection
        self.limit = limit
        self.hide_controls = hide_controls
        self.timer = timer

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen. Here, we use the title that the user gave.
        """
        return self.header or u"Carousel portlet"


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('carousel.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)

    @property
    def available(self):
        return len(self.results())

    def collection_url(self):
        collection = self.collection()
        if collection is None:
            return None
        else:
            return collection.absolute_url()

    def css_class(self):
        header = self.data.header
        normalizer = getUtility(IIDNormalizer)
        return "portlet-carousel-%s" % normalizer.normalize(header)

    @memoize
    def results(self):
        results = []
        collection = self.collection()
        if collection is not None:
            limit = self.data.limit
            if limit and limit > 0:
                # pass on batching hints to the catalog
                results = collection.queryCatalog(batch=True, b_size=limit)
                results = results._sequence
            else:
                results = collection.queryCatalog()
            if limit and limit > 0:
                results = results[:limit]
        return results

    @memoize
    def collection(self):
        collection_path = self.data.target_collection
        if not collection_path:
            return None

        if collection_path.startswith('/'):
            collection_path = collection_path[1:]

        if not collection_path:
            return None

        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        portal = portal_state.portal()
        if isinstance(collection_path, unicode):
            # restrictedTraverse accepts only strings
            collection_path = str(collection_path)
        return portal.restrictedTraverse(collection_path, default=None)

    def use_view_action(self):
        pp = getToolByName(self.context, 'portal_properties', None)
        sp = getattr(pp, 'site_properties', None)
        use_view_action = sp.getProperty('typesUseViewActionInListings', ())
        return use_view_action

    def get_tile(self, obj):
        # When adapter is uesd this means we check whether obj has any special
        # instructions about how to be handled in defined view or interface
        # for multi adapter the same is true except more object than just the
        # obj are check for instructions

        #have to use traverse to make zpt security work
        tile = obj.unrestrictedTraverse("carousel-portlet-view")
        if tile is None:
            return None
        return tile()

    def canSeeEditLink(self):
        provider = self.collection()
        smanager = SecurityManagement.getSecurityManager()
        return smanager.checkPermission(ChangeTopics, provider)

    def editCarouselLink(self):
        provider = self.collection()
        if provider is not None:
            if ICollection.providedBy(provider):
                return provider.absolute_url() + '/edit'
            return provider.absolute_url() + '/criterion_edit_form'
        return None

    def getTimer(self):
        """ return timer in ms"""
        if getattr(self.data, 'timer', None) is not None:
            return int(self.data.timer*1000)
        else:
            return 10000

    def hideHeader(self):
        """ Whether we show the header or not """
        if self.data.hideheader:
            return True
        else:
            return False


class AddForm(base.AddForm):
    form_fields = form.Fields(ICarouselPortlet)
    form_fields['target_collection'].custom_widget = UberSelectionWidget

    label = _(u"Add Carousel Portlet")
    description = _(u"This portlet display a listing of items from a \
                      Collection as a carousel.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    form_fields = form.Fields(ICarouselPortlet)
    form_fields['target_collection'].custom_widget = UberSelectionWidget

    label = _(u"Edit Carousel Portlet")
    description = _(u"This portlet display a listing of items from a \
                      Collection as a carousel.")



from setuptools import setup, find_packages
import os

version = '1.6'

tests_require = [
    'Products.PloneTestCase',
    'collective.testcaselayer']

setup(name='collective.carousel',
      version=version,
      description="Add-on for having Collections' results presented as a nice carousel",
      long_description=open("README.txt").read() + "\n" +
      open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Framework :: Plone",
          "Programming Language :: Python",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      keywords='plone zope collection carousel',
      author='Jarn [Denys Mishunov]',
      author_email='info@jarn.com',
      url='http://svn.plone.org/svn/collective/collective.carousel',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'archetypes.schemaextender',
          'plone.app.jquerytools',
      ],
      tests_require=tests_require,
      extras_require={'tests': tests_require},
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# Carousel is rendered through a viewlet in IAboveContent
# using items provided by the carousel provider added to the context

from zope.interface import alsoProvides, noLongerProvides

from collective.carousel.browser.viewlets import CarouselViewlet
from collective.carousel.testing import ICustomType
from collective.carousel.tests.base import TestCase

# default test query
query = [{
    'i': 'portal_type',
    'o': 'plone.app.querystring.operation.selection.is',
    'v': ['Document', 'Event', 'News Item']
}]


class ViewletTestCase(TestCase):

    def afterSetUp(self):
        """Set up the carousel Collection and some dummy objects"""

        self.setRoles('Manager')
        self.folder.invokeFactory('Collection', 'collection')
        collection = getattr(self.folder, 'collection')
        collection.setQuery(query)

        field = self.folder.Schema().getField('carouselprovider')
        field.set(self.folder, collection)

        # add a few objects
        self.folder.invokeFactory('Document', 'carousel-doc')
        self.folder.invokeFactory('News Item', 'carousel-news-item')
        self.folder.invokeFactory('Event', 'carousel-event')

    def test_viewlet_is_available(self):
        request = self.app.REQUEST
        context = self.folder
        viewlet = CarouselViewlet(context, request, None, None)
        self.failUnless(viewlet)

    def test_multiple_providers(self):
        collections = []
        for i in range(3):
            self.folder.invokeFactory('Collection', 'collection_%s' % i)
            collection = getattr(self.folder, 'collection_%s' % i)
            query = [{
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['Document']
            }]
            collection.setQuery(query)
            collections.append(collection)
        field = self.folder.Schema().getField('carouselprovider')
        field.set(self.folder, tuple(collections))

        viewlet = CarouselViewlet(self.folder, self.app.REQUEST, None, None)

        self.failUnless(len(viewlet.getProviders()) >= 3)

    def test_viewlet_rendering(self):
        # add a few documents
        for i in range(10):
            self.folder.invokeFactory('Document', 'document_%s' % i)
            getattr(self.folder, 'document_%s' % i).reindexObject()

        collection_num_items = len(self.folder.collection.results())
        # We better have some documents in the collection's results
        self.failUnless(collection_num_items >= 10)

        field = self.folder.Schema().getField('carouselprovider')
        # technically the following checkup is done in test_field, but we
        # better check again
        self.assertEqual(field.get(self.folder), [self.folder.collection])

        viewlet = CarouselViewlet(self.folder, self.app.REQUEST, None, None)

        # first check getProviders()
        self.assertEqual(viewlet.getProviders(), [self.folder.collection])

        # check results(). We get not more than 7 items even though the
        # collection returns >=7 results, don't we?
        self.failUnless(len(viewlet.results(viewlet.getProviders()[0])) == 7)
        results = [result.id for result in
                   viewlet.results(viewlet.getProviders()[0])]
        doc_ids = [id for id in self.folder.contentIds()[:7]
                   if 'document' in id]
        for doc_id in doc_ids:
            self.failUnless(doc_id in results)

        # Test that we get correct tiles in the carousel
        for result in viewlet.results(viewlet.getProviders()[0]):
            item_type = result.portal_type
            if item_type == 'Document':
                self.failUnless('<p>This is a PAGE tile</p>' in
                                viewlet.get_tile(result.getObject()))
            if item_type == 'Event':
                self.failUnless('<p>This is a DEFAULT tile</p>' in
                                viewlet.get_tile(result.getObject()))
            if item_type == 'News Item':
                self.failUnless('<p>This is a NEWS ITEM tile</p>' in
                                viewlet.get_tile(result.getObject()))

        # Now we apply new custom tile registration for event object
        event = getattr(self.folder, 'carousel-event')
        alsoProvides(event, ICustomType)
        self.failIf('<p>This is a DEFAULT tile</p>' in viewlet.get_tile(event))

        # We revert event to standard state (without custom tile registration)
        noLongerProvides(event, ICustomType)
        # We should get our DEFAULT tile again
        self.failIf('<p>This is a CUSTOM DEFAULT tile</p>' in
                    viewlet.get_tile(event))

    def test_edit_carousel_link(self):
        viewlet = CarouselViewlet(self.folder, self.app.REQUEST, None, None)
        carousel_criteria = self.folder.collection.absolute_url() + \
            '/edit'
        self.assertEqual(viewlet.editCarouselLink(viewlet.getProviders()[0]),
                         carousel_criteria)

        # Check whether anonymous users get "edit" link. First check that
        # Manager gets it:
        self.failUnless(viewlet.canSeeEditLink(viewlet.getProviders()[0]))

        # Then we switch user to Anonymous:
        self.setRoles([])
        self.logout()
        self.failIf(viewlet.canSeeEditLink(viewlet.getProviders()[0]))


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from Products.Five.testbrowser import Browser
from Testing.ZopeTestCase import Sandboxed
from Products.PloneTestCase import PloneTestCase as ptc
# from collective.carousel import testing

ptc.setupPloneSite()


class TestCase(Sandboxed, ptc.PloneTestCase):
    """ Base class used for test cases """


class FunctionalTestCase(ptc.FunctionalTestCase):
    """ Base FunctionalTestCase for collective.carousel """

    def getBrowser(self, loggedIn=True):
        """ instantiate and return a testbrowser for convenience """
        browser = Browser()
        if loggedIn:
            user = ptc.default_user
            pwd = ptc.default_password
            browser.addHeader('Authorization', 'Basic %s:%s' % (user, pwd))
        return browser
